--[[
	
	Agent Handler
	Handles all generic agent features and knows all agents
	
	- contains some constant agent data (and the unnamed "agents" table)
	
	- triggers hooks in agents/<file> folder
	- updates agent data of user data
	
	
	
]]--

--[[
	CONSTANT ARRAYS
]]
agent = {}



--[[
	Static list loaded from main.lua, containing all different agent-ids
	
	Example: (Order may vary; depends on main.lua)
	Agent.names = {
		"Phoenix",
		"Jett",
		"Chamber",
	}

]]
agent.names = {}


--[[
	Static list loaded from main.lua, containing all different agent-data
	
	Agent.agents = {
		["Phoenix"] = <complex phoenix table> -- content from agents/Phoenix.lua
		
	}

]]
agents = {}


--[[
	Agent.currentAgent = {
		//(name e Agent.names)
		[pid] = name,	
		[13] = "veyne",
		[pid] = <agent id> ,
	}
]]--
agent.current = {}


--[[
	Used to track player data while the game is running.
	
	Maybe a player with id: 13 as phoenix did ult, so the data might be:
	
	agent.data = {
		[13] = {
			ult = {
				info = { 132, 168, 100, 100 } -- x,y,health,armor see phoenix.lua
				img = 1	-- imgId, see phoenix.lua
			},
		}
	
		[id] = {
			-- if this is another agent, it might be completly different data in here!
			<agent data>
		}
	}
]]
agent.data = {}




--[[
	Selected a new agent
]]
function agent.select(id, name)
	if (env.state < 10 or player(id,"health") <= 0) then
		agent.current[id] = name
		user.createSet(id)
		agent.data[id] = {}
		agents[name].onSelect(agent.data[id], id)
	else
		msg2(id,conf.chatWarning.."Agent selected failed")
	end
end

--[[
	Call this when player presses F2-F4
]]
function agent.triggerUtil(pid, utilKey)
	
	local pset = user.getSet(pid)
	local util = agents[agent.current[pid]].utils[utilKey]
	
	if (pset.utils[utilKey].cooldown > 0) then
		return "Still on Cooldown"
	end
	
	if (pset.utils[utilKey].usesLeft == 0) then
		return "No more uses"
	end
	
	if (pset.mana < util.manaCost) then
		return "Not enough mana"
	end

	if (cc.isAffected(pid, "SILENCE")) then
		return "Silenced"
	end
	
	-- Ult
	if (utilKey == 3) then
		if (pset.ultPoints < util.ultCost) then
			return "Not enough ULT Points"
		else
			pset.ultPoints = 0
			agent.hud.updateUltPts(pid)
		end
	end
	
	
	-- isTriggerable = true ALWAYS follows up with a onTrigger()
	if (util.isTriggerable) then
		if (not (util.isTriggerable(agent.data[pid], pid))) then
			return "Utility locked";
		end
	end
	
	-- Set cooldown
	pset.utils[utilKey].cooldown = util.cooldown
	
	-- Set uses left
	pset.utils[utilKey].usesLeft = pset.utils[utilKey].usesLeft - 1
	
	if (pset.utils[utilKey].usesLeft <= 0) then
		agent.hud.drawBorder(pid,utilKey,"NOUSE")
	end
	
	-- Set mana cost
	pset.mana = pset.mana - util.manaCost
	
	-- Trigger ability
	pset.utils[utilKey].isActive = not pset.utils[utilKey].isActive
	
	-- Trigger Ability (depends on ability kind)
	conf.utilKind[util.kind].onTrigger(pid, utilKey, util)
	
	-- Trigger CHook
	cpool.trigger("agentutil",pid,utilKey)
end


function agent.onUntrigger(pid, utilKey, util)
	if (env.state >= 10) then
		user.getSet(pid).utils[utilKey].isActive = false
		util.onUntrigger(agent.data[pid], pid)
	end
end
--[[
	Reduce util cooldown
	Increase mana,
	do what to do after time
]]
function agent.update()
	-- Update for all agents
	for _,pid in ipairs(player(0,"tableliving")) do
		if (agent.current[pid]) then -- player has agent selected
			local pset = user.getSet(pid)
		
			-- Reduce Cooldowns of all abilities
			for _,utilKey in ipairs(conf.user.utilKeys) do
				local cooldown = pset.utils[utilKey].cooldown
				if (cooldown > 0) then
					pset.utils[utilKey].cooldown = cooldown - conf.user.cooldownRegain
				end
				if (player(pid,"bot") == false) then 
					agent.hud.update(pid, utilKey)
				end
			end
			
			-- Increase Mana
			if (pset.mana < conf.user.manaMax) then
				pset.mana = pset.mana + conf.user.manaRegain
			end
			
		end
	end
	
end



-- Hooklist
agent.hlist = {
	{agent.update, "update"},
}

function agent.start()
	for _,pid in ipairs(player(0,"tableliving")) do
		--
		if (player(pid,"bot") == false) then -- !!!!!!! TODO wegmachen wenn bots fertig
			
	
		if (player(pid,"bot") == false) then
			agent.hud.start(pid)
			-- Redraw Icon
			agent.hud.drawIcon(pid)
		end
		
		-- Set spawn weapons
		if (agents[agent.current[pid]].onSpawn) then
			local wpns = agents[agent.current[pid]].onSpawn(agent.data[pid], pid)
			if (wpns) then
				for part in string.gmatch(wpns, "([^,]+)") do -- split on ,
					if (part ~= 'x') then
						parse("equip "..pid.." "..part)
					end
				end
			end
		end
		
		-- Set health, speed
		parse("setmaxhealth "..pid.." "..agents[agent.current[pid]].health)
		parse("sethealth "..pid.." "..agents[agent.current[pid]].health)
		parse("speedmod "..pid.." "..agents[agent.current[pid]].speed)
		
		
		
		end -- !!!!!!!!! TODO wegmachen wenn bots fertig
	end
	hooker.normal:Set(agent.hlist)
end
function agent.stop()
	for _,pid in ipairs(player(0,"tableliving")) do
		if (player(pid,"bot") == false) then
			agents[agent.current[pid]].onSelect(agent.data[pid], pid)
			agent.hud.stop(pid)
			agent.info.stop(pid)
		end
	end
	hooker.normal:Unset(agent.hlist)
end

hooker.normal:Hook(agent.start,"endbuy")
hooker.normal:Hook(agent.stop,"endround")

