
--[[
	Debug file
	
	- creates the F3 menu to manipulate game state (like: unlimited ultimate, etc)
	- prints infos in console
	
	- cool functions like setRoundStart()
	
]]


debug = {}
local valo = "\169030030030VALO: "






--[[
	Sets round 4v5 with 8 random bots and random agent class
	
]]
function debug.setRoundTest(botAmount, classes)
	botAmount = botAmount or 8
	classes = classes or agent.names
	
	
	msg(valo.." New test round with "..botAmount.." bots.")
	
	if (not bots) then
		msg(valo.."Did not load valo bots. They will not use utility and stuff")
		msg(valo.."Bots not scripted yet in current version!")
	end
	
	
	
	local players = {}
	
	-- kick all bots
	for _,v in pairs(player(0,"table")) do
		if (player(v,"bot")) then
			parse("kick "..v)
		else
			players[#players] = v;
			--parse("makespec "..v)
		end
	end
	
	-- add bots
	for _=1,botAmount do
		parse("bot_add")
	end
	
	parse("sv_restart 2")
end

--[[
	
	MENUS
	
]]

menus["DTEST"] = {
	title = "Test Modules Menu",
	items = {
		-- FILLED BY INIT!
	},

}

menus["DCC"] = {
	title = "CC Test Menu",
	items = {
		-- FILLED BY INIT!
	}
}

menus["DRR"] = {
	title = "Round Manipulation",
	items = {
		{"Bot Test","5v5 fill with bots",function(id) debug.setRoundTest() end },
		{"Round Restart","",function(id) parse("sv_restart 3") end },
		{"No Bots","Kick all Bots",
			function(id) 
				for _,v in pairs(player(0,"table")) do
					if (player(v,"bot")) then
						parse("kick "..v)
					end
				end
			end,
		},
		{"Kill all","",
			function(id)
				for _,v in ipairs(player(0,"tableliving")) do
					if (v ~= id) then
						parse("killplayer "..v)
					end
				end
			end
		},
		{"Kill all Bots","",
			function(id)
				for _,v in ipairs(player(0,"table")) do
					if (player(v,"bot")) then
						parse("killplayer "..v)
					end
				end
			end
		
		},
		{"Spawn Clutch","",
			function(id)
				parse("spawnplayer "..id.." 400 400")
				parse("equip "..id.." 82")
				parse("equip "..id.." 85")
				parse("equip "..id.." 46")
				parse("setweapon "..id.." 46")
				parse("sethealth "..id.." 33")
			end
		},
		{"Bot freeze","",
		function(id)
			parse("bot_freeze")
		end
		},
		{"Give Max Ult Points","",
		function(id)
			user.getSet(id).ultPoints = 100
		end
		},
	} 
}


menus["DMAIN"] = {
	title = "Debug Menu",
	items = {
		{"CC Test","Test Crowd Control",function(id) unimenu(id,true,"DCC",1) end},
		{"Round Restart","",function(id) unimenu(id,true,"DRR",1) end},
		{"Test features","",function(id) unimenu(id,true,"DTEST",1) end},
		{"Toggle User Info","",function(id) debug.toggleInfo(id) end},
		{"Generate Content","Servertransfer.lst Agentinfo (temp Folder!)", function(id) content.generateAll() end},
	},
}


debug.display = {}


function debug.toggleInfo(id)
	
	if (debug.display[id]) then
		debug.display[id]= false
		debug.hideInfo(id)
	else
		debug.display[id] = true
		if (not agent.current[id]) then
			msg2(id,"[DEBUGINFO] You are not an agent!")
			return
		end
		
		debug.showInfo(id)
	end	
end


function debug.notice()
	for k,v in pairs(debug.display) do
		if (v) then
			debug.showInfo(k)
		end
	end
end


--[[
	
	INIT

]]
function debug.init(dirTests)

	hooker.normal:Set({
		{debug.say, "say"},
		{debug.onKey, "key"},
		{debug.notice, "notice"},
	})
	addbind(conf.debugKey)
	
	
	-- CC Menu
	for kind,ccs in pairs(cc.kinds) do
		menus["DCC"].items[#menus["DCC"].items+1] = { kind, ccs.desc, function(id) cc.trigger(id,kind,4500) end}
	end
	
	-- Dofile Content Creator!
	dofile(conf.path.."content.lua")
		
	-- Test Menu
	--print("\169255255255Dofiling all tests. Tests are made because debug=true")
	--debug.testload(dirTests)
	--print("\169255255255Dofiling all tests completed successfully!")
	
end


debug.tests = {}

function debug.testload(dirTests)
	for name in io.enumdir(dirTests) do
		if name ~= "." and name ~= ".." then
			
			-- Cut file extension
			for part in string.gmatch(name, "([^.]+)") do -- split on .
				name = part
				break
			end
			
			
			-- Load test file
			print("[DEBUG] Dofile Test: "..dirTests..name)
			
			local file = assert(loadfile(dirTests..name..".lua"));
			file = assert(file())
			
			--dofile(dirTests..name..".lua")
			
			print("[DEBUG] Dofile Finish: "..dirTests..name)
			debug.tests[file.desc] = file
			--table.insert(debug.tests, file)
			-- Load test menu
			menus["DTEST"].items[#menus["DTEST"].items+1] = { 
				name, file.desc, 
				function(id)
					debug.tests[file.desc].test(id)
				end 
			}
		end
	end
end

function debug.onKey(pid, key, state)
	if (state == 1) then
		if (util.isAdmin(pid)) then
			unimenu(pid, true,"DMAIN",1)
		end
	end
end

function debug.say(id, txt)
	if (txt=="!debug") then
		unimenu(id, true,"DMAIN",1)
	
	end

end

