--[[
	VALO / environment.lua
	
	All neccessary input/output, that doesnt belong to the CORE valo gamemode and also
	handling gamestates (triggering gamestate hooks like endbuy, etc)
	In here you find:
	- Pick an Agent Menu
	- msg hints, !say commands
	- user data cs2dhooks regarding leave, join
	- triggering most custom hooks like endbuy, statechange
]]




env = {}

--[[ 
	
	2 = endround
	3 = startround/buyphase
	
	10 = live match (no buyphase)
	
]]
env.state = 1

function env.init()
	env.fillMenus()
	hooker.normal:Set({
		{env.startround,"startround"},
		{env.join,"join"},
		{env.leave,"leave"},
		{env.spawn,"spawn"},
		{env.say,"say"},
		{env.endround, "endround"},
		{env.serveraction,"serveraction"},		-- ABILITY - Addhook Command
		{env.kill, "kill"},
		{env.die, "die"},
	})

	
	
end

env.isBuytime = false

--[[
	ABILITY - Call "env.selectAgent(id) when pressing Serveraction Keys
]]
function env.serveraction(id,b)
	if (player(id,"health") <= 0) then
		env.selectAgent(id)
		return;
	end

	if (env.state >= 10) then
		local err = agent.triggerUtil(id,b)
		if (err) then
			msg2(id,conf.chatWarning..err)
		end
	else
		msg2(id,conf.chatWarning.."New Round didnt start yet")
	end
end

function env.kill(src, vic)
	if (agent.current[src]) then
		local isUltReady = false
		user.getSet(src).ultPoints = user.getSet(src).ultPoints + 1
		
		if (user.getSet(src).ultPoints == agents[agent.current[src]].utils[3].ultCost) then	
			isUltReady = true
		end
		
		if (user.getSet(src).ultPoints > agents[agent.current[src]].utils[3].ultCost) then
			user.getSet(src).ultPoints = agents[agent.current[src]].utils[3].ultCost
		end
		
		cpool.trigger("agentkill", src)
		
		if (isUltReady) then
			cpool.trigger("agentultready", src)
		end
	end
end

function env.die(vic, src)
	if (agent.current[vic]) then
		agent.hud.stop(vic)
		agent.info.stop(vic)
		cpool.trigger("agentdie", vic)
	end

end

function env.startround()
	env.isBuytime = true;
	env.changestate(3)
	for _,pid in ipairs(player(0,"tableliving")) do
		if (agent.current[pid]) then
			user.startround(pid)
		end
	end
	
	
	timerEx(tonumber(game("mp_buytime"))*60000,"env.endbuy",1) -- minutes -> milliseconds
end

function env.changestate(newState)
	if (env.state ~= newState) then
		env.state = newState
		cpool.trigger("statechange")
	end
end

function env.endround()
	env.changestate(2)
	env.isBuytime = false
end




function env.endbuy()
	if (env.isBuytime) then
		-- only trigger when endround() didnt occur yet
		env.changestate(10)
		env.isBuytime = false
		for _, pid in pairs(player(0,"tableliving")) do
			if (player(pid,"bot") == false) then
				if (not agent.current[pid]) then
					parse("killplayer "..pid)
					msg2(pid,"You NEED to select an Agent class!")
					env.selectAgent(pid)
				end
			end
		end

		parse("trigger valo")
		parse("trigger buyb")
		cpool.trigger("endbuy")
	end
end

function env.join(id)
	--user.createSet(id)
	util.msg2(id, "\169255255255", env.greeting)
end

function env.leave(id)
	user.saveSet(id)
	agent.hud.stop(pid)
end

function env.noEntityTrigger()
	msg("TRIGGER DENIED")
	return 1;
end


env.greeting = {
	"--- WELCOME TO VALO2D ---",
	"You have to select your agent with !class",
	"During BUYTIME or while dead!",
	"Press F2, F3, F4 to use your abilities",
	"Use !help, when you selected your agent class to get more information about your agent",
}


env.bindInfo = {
	"You better bind F1, F2, F3 to better keys :D",
	"[ESC] -> Options -> Controls -> Menu -> Change bind for Server Action N",


}




function env.say(id,txt)
	if (txt=="!class") then
		if (env.state < 10) then
			env.selectAgent(id)
		else
			if (player(id,"health") > 0) then 
				msg2(id,"You cant do that right now...")
			else
				
				env.selectAgent(id)
			end
		end
	elseif (txt=="!help") then
		if (agent.current[id]) then
			util.msg2(id, "\169255255255", agents[agent.current[id]].explanation)
		else
			util.msg2(id, "\169255255255", env.greeting)
		end
	elseif (txt=="!bind") then
		util.msg2(id, "\169255255255", env.bindInfo)
	end
end


function env.spawn(id)
	if (not agent.current[id]) then
		if (env.state < 10) then
			msg2(id,conf.chatWarning.."Please select an agent with !class")
			unimenu(id, true, "GAGENT",1)
		end
	else
		if (env.state >= 10) then
			agent.hud.start(id)
		end
	end
end




function env.selectAgent(pid)
	unimenu(pid, true, "GAGENT",1)
end


--[[
	MENU
]]--
menus["GAGENT"] = {
	title = "Pick an Agent Class",
	items = {
		-- FILLED BY INIT
	}
}


function env.fillMenus()
	-- Fill Agent Select Menu 
	for name,agentData in pairs(agents) do
		-- Agent itself
		menus["GAGENT"].items[#menus["GAGENT"].items+1] = { 
			agentData.name, 
			agentData.desc, 
			function(id)
				if (env.state < 10) then
					agent.select(id, name)
					msg2(id,"\169000255000You chose "..agentData.name.." as Agent!"); 
					util.msg2(id,"\169255255255",agents[name].utils[1].explanation)
					util.msg2(id,"\169255255255",agents[name].utils[2].explanation)
					util.msg2(id,"\169255255255",agents[name].utils[3].explanation)
				else
					msg2(id,conf.chatWarning.." Agent Selection not possible ~ LIVE Round!")
				end
			end
		}
		
		-- Abilities
		menus["GAGENT"].items[#menus["GAGENT"].items+1] = { 
			"([F2] "..agentData.utils[1].name..")", 
			"[F3] "..agentData.utils[2].name, 
			function(id) 
				
			end
		}
		
		-- Ultimate
		menus["GAGENT"].items[#menus["GAGENT"].items+1] = { 
			"([F4] "..agentData.utils[3].name..")", 
			agentData.utils[3].desc, 
			function(id) 
				
			end
		}
	end

end






env.init()

