--[[
	Hooker pool
	Please use hooker.lua
	- stores all hook functions as functions
	- calls stored function on hook
	
	Proxy Hook implementation
	https://unrealsoftware.de/forum_posts.php?post=432264
]]

hpool = {}


--[[
	Either you put data like this:
//["string"] = { func1, func2 , ... } --> normal addhook call param=(unpack(arg))

	Or like this: (func inside another table)
//["string"] = { {func1}, {func2}, ... }  --> named addhook call with param=(hookname, unpack(arg))



	
]]			


hpool.hooks = {}


--[[
	Normal 
]]
function hpool.add(func, hook, hooker, data_hooker)
	local pool = hpool.hooks[hook]
	
	if (not pool) then
		hpool.hooks[hook] = {}
		addhook(hook, "hpool.H"..hook, 1000)
	end
	
	--print("HPOOL Add: f("..tostring(func)..") h("..hook.."), type("..tostring(hooker))
	table.insert(hpool.hooks[hook], {fptr = func, data = data_hooker, hwrap = hooker})
	
	
end

function hpool.remove(func, hook)
	local pool = hpool.hooks[hook]
	if (not pool) then return true end -- war noch nix
	if (#pool == 1) then
		if (pool[1].fptr == func) then -- must be 1st index, ipairs array
			freehook(hook, "hpool.H"..hook)
			hpool.hooks[hook] = nil
			return;
		end
	end
	for pIndex, data in pairs(pool) do
		if (data.fptr == func) then
			table.remove(hpool.hooks[hook], pIndex)
		end
	end
end




--[[
	Generic Call function
]]--
function hpool.callAll(hook, ...)
	local pool = hpool.hooks[hook]
	local result
	local guess
--	if (not pool) then print("***********hpool Called nil!?**********") return 1 end -- 
	for _,entry in pairs(hpool.hooks[hook]) do
		guess = entry.hwrap:Call(entry.fptr, entry.data,hook,arg)
		if (guess) then
			result = guess
		end
		
	end
	if (result) then
		return result
	end
end


--[[
	Proxy Hooks 
	https://unrealsoftware.de/forum_posts.php?post=432264
]]

function hpool.Halways(...) return hpool.callAll("always", unpack(arg)) end
function hpool.Hassist(...) return hpool.callAll("assist", unpack(arg)) end
function hpool.Hattack(...) return hpool.callAll("attack", unpack(arg)) end
function hpool.Hattack2(...) return hpool.callAll("attack2", unpack(arg)) end
function hpool.Hbombdefuse(...) return hpool.callAll("bombdefuse", unpack(arg)) end
function hpool.Hbombexplode(...) return hpool.callAll("bombexplode", unpack(arg)) end
function hpool.Hbombplant(...) return hpool.callAll("bombplant", unpack(arg)) end
function hpool.Hbreak(...) return hpool.callAll("break", unpack(arg)) end 
function hpool.Hbuild(...) return hpool.callAll("build", unpack(arg)) end
function hpool.Hbuildattempt(...) return hpool.callAll("buildattempt", unpack(arg)) end
function hpool.Hbuy(...) return hpool.callAll("buy", unpack(arg)) end
function hpool.Hclientdata(...) return hpool.callAll("clientdata", unpack(arg)) end
function hpool.Hclientsetting(...) return hpool.callAll("clientsetting", unpack(arg)) end
function hpool.Hcollect(...) return hpool.callAll("collect", unpack(arg)) end
function hpool.Hconnect(...) return hpool.callAll("connect", unpack(arg)) end
function hpool.Hconnect_attempt(...) return hpool.callAll("connect_attempt", unpack(arg)) end
function hpool.Hconnect_initplayer(...) return hpool.callAll("connect_initplayer", unpack(arg)) end
function hpool.Hdie(...) return hpool.callAll("die", unpack(arg)) end
function hpool.Hdisconnect(...) return hpool.callAll("disconnect", unpack(arg)) end
function hpool.Hdominate(...) return hpool.callAll("dominate", unpack(arg)) end
function hpool.Hdrop(...) return hpool.callAll("drop", unpack(arg)) end
function hpool.Hendround(...) return hpool.callAll("endround", unpack(arg)) end
function hpool.Hflagcapture(...) return hpool.callAll("flagcapture", unpack(arg)) end
function hpool.Hflagtake(...) return hpool.callAll("flagtake", unpack(arg)) end
function hpool.Hflashlight(...) return hpool.callAll("flashlight", unpack(arg)) end
function hpool.Hhit(...) return hpool.callAll("hit", unpack(arg)) end
function hpool.Hhitzone(...) return hpool.callAll("hitzone", unpack(arg)) end
function hpool.Hhostagedamage(...) return hpool.callAll("hostagedamage", unpack(arg)) end
function hpool.Hhostagekill(...) return hpool.callAll("hostagekill", unpack(arg)) end
function hpool.Hhostagerescue(...) return hpool.callAll("hostagerescue", unpack(arg)) end
function hpool.Hhostageuse(...) return hpool.callAll("hostageuse", unpack(arg)) end
function hpool.Hhttpdata(...) return hpool.callAll("httpdata", unpack(arg)) end
function hpool.Hitemfadeout(...) return hpool.callAll("itemfadeout", unpack(arg)) end
function hpool.Hjoin(...) return hpool.callAll("join", unpack(arg)) end
function hpool.Hkey(...) return hpool.callAll("key", unpack(arg)) end
function hpool.Hkill(...) return hpool.callAll("kill", unpack(arg)) end
function hpool.Hleave(...) return hpool.callAll("leave", unpack(arg)) end
function hpool.Hlog(...) return hpool.callAll("log", unpack(arg)) end
function hpool.Hmapchange(...) return hpool.callAll("mapchange", unpack(arg)) end
function hpool.Hmenu(...) return hpool.callAll("menu", unpack(arg)) end
function hpool.Hminute(...) return hpool.callAll("minute", unpack(arg)) end
function hpool.Hmove(...) return hpool.callAll("move", unpack(arg)) end
function hpool.Hmovetile(...) return hpool.callAll("movetile", unpack(arg)) end
function hpool.Hms100(...) return hpool.callAll("ms100", unpack(arg)) end
function hpool.Hname(...) return hpool.callAll("name", unpack(arg)) end
function hpool.Hobjectdamage(...) return hpool.callAll("objectdamage", unpack(arg)) end
function hpool.Hobjectkill(...) return hpool.callAll("objectkill", unpack(arg)) end
function hpool.Hobjectupgrade(...) return hpool.callAll("objectupgrade", unpack(arg)) end
function hpool.Hparse(...) return hpool.callAll("parse", unpack(arg)) end
function hpool.Hprojectile(...) return hpool.callAll("projectile", unpack(arg)) end
function hpool.Hprojectile_impact(...) return hpool.callAll("projectile_impact", unpack(arg)) end
function hpool.Hradio(...) return hpool.callAll("radio", unpack(arg)) end
function hpool.Hrcon(...) return hpool.callAll("rcon", unpack(arg)) end
function hpool.Hreload(...) return hpool.callAll("reload", unpack(arg)) end
function hpool.Hsay(...) return hpool.callAll("say", unpack(arg)) end
function hpool.Hsayteam(...) return hpool.callAll("sayteam", unpack(arg)) end
function hpool.Hsayteamutf8(...) return hpool.callAll("sayteamutf8", unpack(arg)) end
function hpool.Hsayutf8(...) return hpool.callAll("sayutf8", unpack(arg)) end
function hpool.Hsecond(...) return hpool.callAll("second", unpack(arg)) end
function hpool.Hselect(...) return hpool.callAll("select", unpack(arg)) end
function hpool.Hserveraction(...) return hpool.callAll("serveraction", unpack(arg)) end
function hpool.Hshieldhit(...) return hpool.callAll("shieldhit", unpack(arg)) end
function hpool.Hshutdown(...) return hpool.callAll("shutdown", unpack(arg)) end
function hpool.Hspawn(...) return hpool.callAll("spawn", unpack(arg)) end
function hpool.Hspecswitch(...) return hpool.callAll("specswitch", unpack(arg)) end
function hpool.Hspray(...) return hpool.callAll("spray", unpack(arg)) end
function hpool.Hstartround(...) return hpool.callAll("startround", unpack(arg)) end
function hpool.Hstartround_prespawn(...) return hpool.callAll("startround_prespawn", unpack(arg)) end
function hpool.Hsuicide(...) return hpool.callAll("suicide", unpack(arg)) end
function hpool.Hteam(...) return hpool.callAll("team", unpack(arg)) end
function hpool.Htrigger(...) return hpool.callAll("trigger", unpack(arg)) end
function hpool.Htriggerentity(...) return hpool.callAll("triggerentity", unpack(arg)) end
function hpool.Hturretscan(...) return hpool.callAll("turretscan", unpack(arg)) end
function hpool.Huse(...) return hpool.callAll("use", unpack(arg)) end
function hpool.Husebutton(...) return hpool.callAll("usebutton", unpack(arg)) end
function hpool.Hvipescape(...) return hpool.callAll("vipescape", unpack(arg)) end
function hpool.Hvoice(...) return hpool.callAll("voice", unpack(arg)) end
function hpool.Hvote(...) return hpool.callAll("vote", unpack(arg)) end
function hpool.Hwalkover(...) return hpool.callAll("walkover", unpack(arg)) end



--[[
hpool.names = {
"always",
"assist",
"attack",
"attack2",
"bombdefuse",
"bombexplode",
"bombplant",
"break",
"build",
"buildattempt",
"buy",
"clientdata",
"clientsetting",
"collect",
"connect",
"connect_attempt",
"connect_initplayer",
"die",
"disconnect",
"dominate",
"drop",
"endround",
"flagcapture",
"flagtake",
"flashlight",
"hit",
"hitzone",
"hostagedamage",
"hostagekill",
"hostagerescue",
"hostageuse",
"httpdata",
"itemfadeout",
"join",
"key",
"kill",
"leave",
"log",
"mapchange",
"menu",
"minute",
"move",
"movetile",
"ms100",
"name",
"objectdamage",
"objectkill",
"objectupgrade",
"parse",
"projectile",
"projectile_impact",
"radio",
"rcon",
"reload",
"say",
"sayteam",
"sayteamutf8",
"sayutf8",
"second",
"select",
"serveraction",
"shieldhit",
"shutdown",
"spawn",
"specswitch",
"spray",
"startround",
"startround_prespawn",
"suicide",
"team",
"trigger",
"triggerentity",
"turretscan",
"use",
"usebutton",
"vipescape",
"voice",
"vote",
"walkover",
}
]]

