--[[
	Hook Wrapper Class
	So the Hooker knows which condition have to be true to trigger
]]



----------------------------------------------------------------------
-- AgentHooker for calling hooks when certain pid in hook on position argi_of_pid equals agentname
-- conf.agenthooker:Hook(myDie,"die",2,"gnuk") - myDie is triggered only when "src" was a 'gnuk'
----------------------------------------------------------------------
AgentHooker = {}
AgentHooker = Hooker:New()

-- is current id correct agent?
function AgentHooker:CheckCall(fptr, data, ...)
	--print("*** "..data.pidindex.." f("..tostring(fptr))
	local pid = arg[data.pidindex or 1]
	if (env.state < 10) then return end
	--print("CHECKCALL: "..pid.." WITH dat("..tostring(data.name))
	if (agent.current[pid] == data.name) then
		--print("AgentHooker Checkcall Success!pid="..pid..", f("..tostring(fptr).."), name="..tostring(data.name))
		return fptr(agent.data[pid],unpack(arg))
	end
end


function AgentHooker:Hook(func, hook, argi_of_pid, agentkind)
	--print("AgentHooker Hooked f("..tostring(func).." h("..hook..") type("..tostring(self))
	self:Add(func, hook, self, {pidindex = argi_of_pid, name = agentkind })	
end


----------------------------------------------------------------------
-- CC Hooker same idea as agent hooker, just for is current CC Kind active
-- cc.kinds["Lifesteal"].attack(data, id) only gets triggered when that id has Lifesteal applied
----------------------------------------------------------------------
-- CC
CCHooker = {}
CCHooker = Hooker:New()

-- is current id affected by correct CC?
function CCHooker:CheckCall(fptr, data, ...)
	
	
	local pid = arg[data.pidindex or 1]
	if (not cc.active[pid]) then return end -- error with custom projectiles on hit hook
	if (cc.active[pid][data.name]) then
	--	print("CC Checkcall success "..pid.." has "..data.name.."")
		return fptr(cc.active[pid][data.name],unpack(arg))
	end
end

function CCHooker:Hook(func, hook, argi_of_pid, cckind)
	--print("CCHooker Hooked f("..tostring(func).." h("..hook..") type("..tostring(self))
	self:Add(func, hook, self, {pidindex = argi_of_pid, name = cckind })	
end


----------------------------------------------------------------------
-- PConditionHooker, hooks only if player(pid,playerValue) == expectedValue
-- agent.hud.start()>> hooker.time:Hook(func,"update",hooker.bot,)
----------------------------------------------------------------------
-- CC
PConditionHooker = {}
PConditionHooker = Hooker:New()

-- is current id affected by correct CC?
function PConditionHooker:CheckCall(fptr, data, ...)
	local pid = arg[data.pidindex or 1]
	if (player(pid,data.playerVal) == data.expValue) then
		return fptr(unpack(arg))
	end
end

function PConditionHooker:Hook(func, hook, argi_of_pid, playerValue, expectedValue)
	self:Add(func, hook, self, {pidindex = argi_of_pid, playerVal = playerValue, expValue = expectedValue })	
end



----------------------------------------------------------------------
----------------------------------------------------------------------
----------------------------------------------------------------------
----------------------------------------------------------------------
----------------------------------------------------------------------
--	Timing, calls hookerKind for every pid on conf.hooktime event
--	timingHooker:Hook(myUpdate, "update", conf.agenthooker,1,"gnuk")
-- 	timingHooker:Hook(mySec,"second", cchooker,1,"Lifesteal") 
----------------------------------------------------------------------
TimingHooker = {}
TimingHooker = Hooker:New()


function TimingHooker:CheckCall(fptr, data, ...)
	local result
	--print("TIMING CALL f("..tostring(fptr).."),typ("..tostring(data.hooker)..") name("..tostring(data.a))
	for _,pid in ipairs(player(0,"tableliving")) do
		result = data.hooker:CheckCall(fptr, data.a, pid)
	end
	return result
end

-- e.g.: (mySec1, "second", agentHooker,"gnuk") for an agent hooker
-- the hookerKind needs to have the "argi_of_pid" argument in :Hook!
function TimingHooker:Hook(func, hook, hookerKind, hookerData)
	--^-print("Timing Hooked f("..tostring(func).." h("..hook..") <"..name.."> type("..tostring(hookerKind))
	self:Add(func, hook, self, {hooker = hookerKind, a = hookerData})
end


