--[[
	Main VALO
	
	- dofile all files in correct order
	- manage dependency dofile
	
	
]]--

main = {}
main.version = "1.0"

--[[
	DEBUG MENU  [false/true]
	false = no debug menu
	true = debug menu, you need to be admin in conf.lua!
]]
main.debug = false


main.files = {}
main.files.prefix = "sys/lua/valo/"
main.files.dependencies = "modules/"
main.files.agents = "agents/"

if (main.debug) then
	main.files.tests = "tests/"
end



main.preAgentOrder = {
	
	"conf.lua",					-- some constant values, like all hud-ids
	"util.lua",					-- basic functions, like pid.isAdmin(pid)
	
	"dependencies.lua",			-- loads dependencys for current agents
	"hooker/dofile.lua",		-- addhooking made easy (or complicated, depends :3)
	
	"user.lua",					-- current data about current cooldowns, mana,
	
	-- TODO: Not included yetmid
	--"projectiles/dofile.lua",		-- projectile system
	--"bots/dofile.lua",			-- Bots using abilities and picking random agent
	
	"agent/dofile.lua",			-- Agent info: Player agent selection, agent List
}

main.postAgentOrder = {
	"cc/dofile.lua",			-- Crowd Control effects
	"env.lua",					-- everything apart the gameplay (select agent menu, get ads, !say)
}

--[[
	- Fill the Agent.names list
	- Load all neccessary dependencies
	- dofile every file in agents/ folder
]]
local function agentLoad()
	local dirAgents = main.files.prefix..main.files.agents
	local dirDependencies = main.files.prefix..main.files.dependencies
	 
	for name in io.enumdir(dirAgents) do
		if name ~= "." and name ~= ".." then
			-- Cut file extension
			for part in string.gmatch(name, "([^.]+)") do -- split on .
				name = part
				break
			end
			
			-- Load dependencies for agent
			dependencies.reload(dirDependencies, name)
			
			-- Load agents/agent file
			print("\169255255255Agent Dofile: "..name)
			dofile(dirAgents..name..".lua");
			if (not agents[name]) then
				print("\169255000000Agent "..name.." not loaded correctly!")
			end
			
			
			-- Dofile name.lua and put agent content into agent mega table
			
			
			-- Call "onInit()" of each agent
			if (agents[name].onInit) then
				agents[name].onInit(agents[name])
			end
		end
	end
end

local function listLoad(list, printInfo, postFix)
	postFix = postFix or ""
	for _,arg in pairs(list) do
		print("\169100255100"..printInfo..arg)
		dofile(main.files.prefix..arg..postFix)
	end
end


local function initDofile()
	parse("clear")
	local timeStart = os.clock()
	print("--- INIT VALO START (Version: "..main.version..") ---")
	
	-- PRE AGENT LOAD
	listLoad(main.preAgentOrder, "PreAgentLoad: ")
	
	-- CONF INITIALIZE
	initgamesettings()
	
	-- BASIC MODULES LOAD
	dependencies.init(main.files.prefix)
	
	-- AGENT & DEPENDENCY LOAD
	agentLoad()
	
	-- POST AGENT LOAD
	listLoad(main.postAgentOrder, "PostAgentLoad: ")
	
	if (main.debug) then
		local info = "\169255069069***VALO Debug Mode***"
		print(info)
		print(info)
		print(info)
		dofile(main.files.prefix.."debug.lua")
		debug.init(main.files.prefix..main.files.tests)
		msg("\169255069069***VALO Debug Mode***")
	end
	
	print("--- INIT VALO END (Time req.: "..os.clock()-timeStart..") ---")
	
end

initDofile()